'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _fieldAttr = require('../field-attr');

var _index = require('../index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

jest.useFakeTimers();

var ValidValidator = function (_Validator) {
  (0, _inherits3.default)(ValidValidator, _Validator);

  function ValidValidator() {
    (0, _classCallCheck3.default)(this, ValidValidator);
    return (0, _possibleConstructorReturn3.default)(this, _Validator.apply(this, arguments));
  }

  return ValidValidator;
}(_index.Validator);

var InvalidValidator = function (_Validator2) {
  (0, _inherits3.default)(InvalidValidator, _Validator2);

  function InvalidValidator() {
    (0, _classCallCheck3.default)(this, InvalidValidator);
    return (0, _possibleConstructorReturn3.default)(this, _Validator2.apply(this, arguments));
  }

  InvalidValidator.prototype.isValid = function isValid() {
    return false;
  };

  return InvalidValidator;
}(_index.Validator);

var v3 = function v3(errors) {
  return function () {
    return new _promise2.default(function (resolve, reject) {
      setTimeout(function () {
        if (errors) {
          reject(errors);
        } else {
          resolve();
        }
      }, 500);
    });
  };
};

describe('FieldAttr', function () {
  var fieldAttr = void 0;
  beforeEach(function () {
    fieldAttr = new _fieldAttr.FieldAttr({}, 'name', '', [], {}, null, {});
  });

  describe('initialization', function () {
    it('create', function () {
      expect(fieldAttr).toBe(fieldAttr);
    });

    it('setValue: should update value', function () {
      fieldAttr.setValue();
      expect(fieldAttr.value).toEqual('');
      fieldAttr.setValue('new-value');
      expect(fieldAttr.value).toEqual('new-value');
    });

    it('setValidators: should update validators', function () {
      fieldAttr.setValidators();
      expect(fieldAttr.validators).toEqual([]);
      fieldAttr.setValidators([1]);
      expect(fieldAttr.validators).toEqual([1]);
    });

    it('setValidatorsOptions: should update validatorsOptions', function () {
      fieldAttr.setValidatorsOptions();
      expect(fieldAttr.validatorsOptions).toEqual({});
      fieldAttr.setValidatorsOptions({ test: 1 });
      expect(fieldAttr.validatorsOptions).toEqual({ test: 1 });
    });

    it('setAsyncValidator: should update asyncValidator', function () {
      fieldAttr.setAsyncValidator();
      expect(fieldAttr.asyncValidator).not.toBeDefined();
      fieldAttr.setAsyncValidator({ test: 1 });
      expect(fieldAttr.asyncValidator).toEqual({ test: 1 });
    });

    it('setAsyncValidatorOptions: should update asyncValidatorOptions', function () {
      fieldAttr.setAsyncValidatorOptions();
      expect(fieldAttr.asyncValidatorOptions).toEqual({});
      fieldAttr.setAsyncValidatorOptions({ test: 1 });
      expect(fieldAttr.asyncValidatorOptions).toEqual({ test: 1 });
    });

    it('setErrors: should update errors', function () {
      fieldAttr.setErrors();
      expect(fieldAttr.errors).toEqual([]);
      fieldAttr.setErrors([1]);
      expect(fieldAttr.errors).toEqual([1]);
    });

    it('setDirty: should update dirty/pristine', function () {
      fieldAttr.setDirty();
      expect(fieldAttr.dirty).toEqual(false);
      expect(fieldAttr.pristine).toEqual(true);
      fieldAttr.setDirty(true);
      expect(fieldAttr.dirty).toEqual(true);
      expect(fieldAttr.pristine).toEqual(false);
    });

    it('setTouched: should update touched', function () {
      fieldAttr.setTouched();
      expect(fieldAttr.touched).toEqual(false);
      fieldAttr.setTouched(true);
      expect(fieldAttr.touched).toEqual(true);
    });

    it('setFocus: should update focused', function () {
      fieldAttr.setFocus();
      expect(fieldAttr.focused).toEqual(false);
      fieldAttr.setFocus(true);
      expect(fieldAttr.focused).toEqual(true);
    });
  });

  describe('validate', function () {
    var v1 = new ValidValidator();
    var v2 = new InvalidValidator();

    it('validate: should get empty errors list', function () {
      fieldAttr.validate();
      expect(fieldAttr.errors).toEqual([]);
      fieldAttr.setValidators([v1]);
      fieldAttr.validate();
      expect(fieldAttr.errors).toEqual([]);
    });

    it('validate: should get multi errors list', function () {
      fieldAttr.setValidators([v2, v2]);
      fieldAttr.validate();
      expect(fieldAttr.errors.length).toEqual(2);

      fieldAttr.setValidators([v1, v2]);
      fieldAttr.validate();
      expect(fieldAttr.errors.length).toEqual(1);
    });

    it('validate: should get single error in list', function () {
      fieldAttr.setValidators([v1, v2, v2]);
      fieldAttr.setValidatorsOptions({ multi: false });
      fieldAttr.validate();
      expect(fieldAttr.errors.length).toEqual(1);
    });

    it('asyncValidate: should get empty errors list', function (done) {
      fieldAttr.setAsyncValidator(v3());
      fieldAttr.asyncValidate(null, function () {
        expect(fieldAttr.asyncErrors.length).toEqual(0);
        done();
      });
      jest.runAllTimers();
    });

    it('asyncValidate: should get errors list', function (done) {
      fieldAttr.setAsyncValidator(v3(['error']));
      fieldAttr.asyncValidate(null, function () {
        expect(fieldAttr.asyncErrors.length).toEqual(1);
        done();
      });
      jest.runAllTimers();
    });

    it('asyncValidate: should validate after static validators', function (done) {
      fieldAttr.setValidators([v2]);
      fieldAttr.setAsyncValidator(v3(['error']));
      fieldAttr.setAsyncValidatorOptions({ validateAfterLocal: true });
      fieldAttr.validate();
      fieldAttr.asyncValidate(null, function () {
        expect(fieldAttr.asyncErrors.length).toEqual(0);
        done();
      });
    });

    it('asyncValidate: should pass validate with incorrect event', function (done) {
      fieldAttr.setAsyncValidator(v3(['error']));
      fieldAttr.setAsyncValidatorOptions({ validateOn: [_fieldAttr.FieldAttr.events.CHANGE] });
      fieldAttr.asyncValidate(null, function () {
        expect(fieldAttr.asyncErrors.length).toEqual(0);
        done();
      });
    });

    it('asyncValidate: should validate with correct event', function (done) {
      fieldAttr.setAsyncValidator(v3(['error']));
      fieldAttr.setAsyncValidatorOptions({ validateOn: [_fieldAttr.FieldAttr.events.CHANGE] });
      fieldAttr.asyncValidate(_fieldAttr.FieldAttr.events.CHANGE, function () {
        expect(fieldAttr.asyncErrors.length).toEqual(1);
        done();
      });
      jest.runAllTimers();
    });

    it('asyncValidate: shouldn\'t validate', function (done) {
      fieldAttr.setAsyncValidator();
      fieldAttr.asyncValidate(null, function () {
        expect(fieldAttr.asyncErrors.length).toEqual(0);
        done();
      });
    });
  });
});
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(ValidValidator, 'ValidValidator', 'E:/Projects/React/git/vb-react-form/src/__tests___/field-attr.spec.js');

  __REACT_HOT_LOADER__.register(InvalidValidator, 'InvalidValidator', 'E:/Projects/React/git/vb-react-form/src/__tests___/field-attr.spec.js');

  __REACT_HOT_LOADER__.register(v3, 'v3', 'E:/Projects/React/git/vb-react-form/src/__tests___/field-attr.spec.js');
}();

;